﻿--------------------------------
--        ScoreMe 3.0.1       --
--   Fluxec on EU Anetheron   --
--------------------------------

----------------------------
-- Initializes Variables  --
----------------------------

ScoreMeSettings = {}; -- Creates settings array.
ScoreMeSettings["ScoreMeDisabled"] = false; -- Sets whether the total ScoreMe should be displayed in tooltip.
ScoreMeSettings["scoreOn"]  = true; -- Sets whether GearScore should be displayed for players.
ScoreMeSettings["itemOn"] = true; -- Sets whether GearScore should be displayed for items.
ScoreMeSettings["ilvlOn"] = true; -- Sets whether the currently equipped average ilevel should be displayed for players.
ScoreMeSettings["boaOn"] = false; -- Sets whether bind on account gear message will be displayed for players.
ScoreMeSettings["missingOn"] = false; -- Sets whether missing gear message will be displayed for players.
ScoreMeSettings["pvpOn"] = false; -- Sets whether PVP gear message will be displayed for players.
ScoreMeSettings["statsOn"] = false; -- Sets whether PVP gear message will be displayed for players.

local ScoreMeVersion = "3.0.1 - Fluxec - EU Anetheron";	-- Sets the version number, author and realm.
local ScoreMeRelease = true;		-- Flag if this is a full release version.
local combatStatus = false;		-- Flag for combat status.
local boa = false;				-- Flag for bind on account gear.
local missing = false;			-- Flag for missing gear.
local pvp = false;				-- Flag for PVP Gear.
	
local textColors = {};			-- Easy access to text coloring.
	textColors["black"]	 = "|cff000000";
	textColors["white"]  = "|cffffffff";
	textColors["gray"]	 = "|cffc9d9d9";
	textColors["yellow"] = "|cffffff00";
	textColors["red"] 	 = "|cffff0000";
	textColors["green"]	 = "|cff1eff00";
	textColors["blue"]	 = "|cff0070dd";
	textColors["purple"] = "|cffa335ee";
	textColors["orange"] = "|cffff8000";

local slotMod = {}
	slotMod["INVTYPE_2HWEAPON"] = 2
	slotMod["INVTYPE_HEAD"] = 1
	slotMod["INVTYPE_NECK"] = 0.5625
	slotMod["INVTYPE_SHOULDER"] = 0.75
	slotMod["INVTYPE_CHEST"] = 1
	slotMod["INVTYPE_ROBE"] = 1
	slotMod["INVTYPE_WAIST"] = 0.75
	slotMod["INVTYPE_LEGS"] = 1
	slotMod["INVTYPE_FEET"] = 0.75
	slotMod["INVTYPE_WRIST"] = 0.5625
	slotMod["INVTYPE_HAND"] = 0.75
	slotMod["INVTYPE_FINGER"] = 0.5625
	slotMod["INVTYPE_TRINKET"] = 0.5625
	slotMod["INVTYPE_CLOAK"] = 0.5625
	slotMod["INVTYPE_WEAPONMAINHAND"] = 1.125
	slotMod["INVTYPE_WEAPONOFFHAND"] = 1.125
	slotMod["INVTYPE_WEAPON"] = 1.125
	slotMod["INVTYPE_HOLDABLE"] = 1.125
	slotMod["INVTYPE_SHIELD"] = 1.125
	slotMod["INVTYPE_RANGED"] = 0.3164
	slotMod["INVTYPE_THROWN"] = 0.3164
	slotMod["INVTYPE_RANGEDRIGHT"] = 0.3164
	slotMod["INVTYPE_RELIC"] = 0.3164

-------------------------
-- Main body of addon. --
-------------------------

-- Handles program events.
function ScoreMeEvents(event) 
	if (event == "PLAYER_REGEN_DISABLED") then -- Set in combat status.
		combatStatus = true;
	end
	if (event == "PLAYER_REGEN_ENABLED") then -- Set out of combat status.
		combatStatus = false;
	end
end

-- Commands for ScoreMe.
function UserCommands(cmdInput) 
 	if (cmdInput == "info") then -- Get Version and Author Information.
		if (ScoreMeRelease) then
			print(textColors.yellow.."ScoreMe Version and Author: "..textColors.green..ScoreMeVersion.." |r.");
			return;
		else
			print(textColors.yellow.."ScoreMe Version and Author: "..textColors.red..ScoreMeVersion.." |r.");
			return;
		end
	end
	if (cmdInput == "enable") then -- Enables the addon.
		ScoreMeSettings["ScoreMeDisabled"] = false;
		print(textColors.yellow.."ScoreMe is now "..textColors.green.."Enabled|r.");
		return;
	end
	if (cmdInput == "disable") then -- Disables the addon.
		ScoreMeSettings["ScoreMeDisabled"] = true;
		print(textColors.yellow.."ScoreMe is now "..textColors.red.."Disabled|r.");
		return;
	end
	if (cmdInput == "help") then -- Display command list for ScoreMe
		print(textColors.yellow.."ScoreMe Commands: |r");
		print(textColors.white.."info - Display's information about ScoreMe. |r");
		print(textColors.white.."enable - Enables ScoreMe tooltip additions. |r");
		print(textColors.white.."disable - Disables ScoreMe tooltip additions. |r");
		print(textColors.white.."boa - Toggles the BoA tooltip feature of ScoreMe. |r");
		print(textColors.white.."missing - Toggles the missing item tooltip feature of ScoreMe. |r");
		--print(textColors.white.."score - Toggles the GearScore feature of ScoreMe. |r");
		print(textColors.white.."scoreme - Toggles the ScoreMe feature of ScoreMe. |r");
		print(textColors.white.."item - Toggles the item tooltip feature of ScoreMe. |r");
		print(textColors.white.."pvp - Toggles the pvp gear tooltip feature of ScoreMe. |r");
		return;
	end
	if (cmdInput == "pvp") then -- Toggles BoA feature.
		if (ScoreMeSettings["pvpOn"]) then
			ScoreMeSettings["pvpOn"] = false;
			print(textColors.yellow.."PVP feature is now "..textColors.red.."Disabled|r.");
		else
			ScoreMeSettings["pvpOn"] = true;
			print(textColors.yellow.."PVP feature is now "..textColors.green.."Enabled|r.");
		end
		return;
	end
	if (cmdInput == "boa") then -- Toggles BoA feature.
		if (ScoreMeSettings["boaOn"]) then
			ScoreMeSettings["boaOn"] = false;
			print(textColors.yellow.."BoA feature is now "..textColors.red.."Disabled|r.");
		else
			ScoreMeSettings["boaOn"] = true;
			print(textColors.yellow.."BoA feature is now "..textColors.green.."Enabled|r.");
		end
		return;
	end
	if (cmdInput == "missing") then -- Toggles missing item feature.
		if (ScoreMeSettings["missingOn"]) then
			ScoreMeSettings["missingOn"] = false;
			print(textColors.yellow.."Missing item feature is now "..textColors.red.."Disabled|r.");
		else
			ScoreMeSettings["missingOn"] = true;
			print(textColors.yellow.."Missing item feature is now "..textColors.green.."Enabled|r.");
		end
		return;
	end
	if (cmdInput == "scoreme") or (cmdInput == "ilvl") then -- Toggles ScoreMe ( GSS ) feature.
		if (ScoreMeSettings["ilvlOn"]) then
			ScoreMeSettings["ilvlOn"] = false;
			print(textColors.yellow.."ScoreMe feature is now "..textColors.red.."Disabled|r.");
		else
			ScoreMeSettings["ilvlOn"] = true;
			print(textColors.yellow.."ScoreMe feature is now "..textColors.green.."Enabled|r.");
		end
		return;
	end
	--if (cmdInput == "score") then -- Toggles GearScore display feature.
		--if (ScoreMeSettings["scoreOn"]) then
			--ScoreMeSettings["scoreOn"] = false;
			--print(textColors.yellow.."Total GearScore feature is now "..textColors.red.."Disabled|r.");
		--else
			--ScoreMeSettings["scoreOn"] = true;
			--print(textColors.yellow.."Total GearScore feature is now "..textColors.green.."Enabled|r.");
		--end
		--return;
	--end
	--if (cmdInput == "item") then -- Toggles GearScore item feature.
		--if (ScoreMeSettings["itemOn"]) then
			--ScoreMeSettings["itemOn"] = false;
			--print(textColors.yellow.."GearScore item tooltip feature is now "..textColors.red.."Disabled|r.");
		--else
			--ScoreMeSettings["itemOn"] = true;
			--print(textColors.yellow.."GearScore item tooltip feature is now "..textColors.green.."Enabled|r.");
		--end
		--return;
	--end
	print(textColors.yellow.."ScoreMe: Unknown Command. Type 'help' for a list of commands.|r")
end

-- Retrieves an items information to calculate it's ScoreMe.
function GetItemScore(iLink) 
	local tempScore = 0;
	local _, _, itemRarity, itemLevel, _, _, _, _, itemEquip = GetItemInfo(iLink);
	if (IsEquippableItem(iLink)) and (slotMod[itemEquip]) then -- Filters out non equipable items.
		if (itemLevel > 1) and (itemRarity > 1) then -- Calculates a temporary score based on ilvl and quality.
			tempScore = itemLevel * (1 + .1 * (itemRarity - 2));
		elseif (itemRarity == 7) and (itemLevel == 1) then -- Adjust for WotLK BoA gear
			tempScore = 187 * 1.1;
			boa = true;
		end
		if (tempScore > 1) then
			tempScore = tempScore * (slotMod[itemEquip])
		end
	else
		return 0,0;
	end
	return floor(tempScore), itemLevel;
end

-- Attempts to calculate the unit's average item level( LevelMe ) and total ScoreMe.
function GetPlayerScore(unit) 
	boa, pvp, missing = false, false,false;
	local ScoreMe = 0;
	local ilvl = 0;
	local ilvlAdd = 0;
	local equipped = 0;
	local itemScoreTable = {};
	if (UnitIsPlayer(unit)) then
		local _, targetClass = UnitClass(unit);
		for i = 1, 18 do -- Walks through the targets items to get individual scores.
			if (i ~= 4) then
				local iLink = GetInventoryItemLink(unit, i);
				if (iLink) then
					itemScoreTable[i], ilvlAdd = GetItemScore(iLink);
					if ScoreMeSettings["pvpOn"] then
						CheckItemStats(iLink);
					end
					ilvl = ilvl + ilvlAdd;
					equipped = equipped + 1;
				elseif (i ~= 17) then
					missing = true;
					itemScoreTable[i] = 0;
				else
					itemScoreTable[i] = 0;
				end
			end
		end
		if  (targetClass == "HUNTER") then -- Adjusts for Hunter's ranged weapons.
			itemScoreTable[16] = itemScoreTable[16] * .281;
			itemScoreTable[17] = itemScoreTable[17] * .281;
			itemScoreTable[18] = itemScoreTable[18] * 6.321;
		end
		if (GetInventoryItemLink(unit, 17)) then -- Adjusts for Titan's Grip.
			local _, iLink, itemRarity, itemLevel, _, _, _, _, itemEquip = GetItemInfo(GetInventoryItemLink(unit, 17));
			if (itemEquip == "INVTYPE_2HWEAPON") then
				itemScoreTable[16] = itemScoreTable[16] * .5;
				itemScoreTable[17] = itemScoreTable[17] * .5;
			end
		end
		for j = 1, 18 do -- Adds up total Score.
			if  (itemScoreTable[j]) then
				ScoreMe = ScoreMe + itemScoreTable[j];
			end
		end
	end
	ClearInspectPlayer(); -- Clears inspection data.
	return floor(ScoreMe), floor(ilvl / equipped);
end

-- Checks the stats on the currently equipped gear, flags important stats.
function CheckItemStats(iLink)
	local statTable = {};
	statTable = GetItemStats(iLink);
	if (statTable["ITEM_MOD_RESILIENCE_RATING_SHORT"]) then
		pvp = true;
	end
end

-- Checks if you are moused over someone and adds output lines to tooltip if nessecary.
function ScoreMeSetUnit() 
	if (combatStatus) or (ScoreMeSettings["ScoreMeDisabled"]) then  -- Checks whether in combat & that addon is enabled before proceeding.
		return; 
	end
	local _, unit = GameTooltip:GetUnit();
	local unitScore, unitilvl = 0,0;
	if not (unit) or not (UnitIsPlayer(unit)) or not (CanInspect(unit)) then
		return;
	elseif (UnitIsUnit(unit,"player")) then -- Checks if you are targeting yourself
		unitScore, unitilvl = GetPlayerScore("player");
	elseif not (InspectFrame and InspectFrame:IsShown()) then -- Checks that you do not have the inspection window open.
		NotifyInspect(unit); unitScore, unitilvl = GetPlayerScore(unit);
	end
	--if not (ScoreMeSettings["ScoreMeDisabled"]) then
		--if (ScoreMeSettings["scoreOn"]) then  -- Adds GearScore & color coding to tooltip.
			--if (unitScore > 1) and (unitScore < 2800) then -- Poor.
				--GameTooltip:AddLine(textColors.gray.."GearScore: "..unitScore,.62,.62,.62);
			--elseif (unitScore >= 2800) and (unitScore < 3500) then -- Common.
				--GameTooltip:AddLine(textColors.white.."GearScore: "..unitScore,1,1,1);
			--elseif (unitScore >= 3500) and (unitScore < 4300) then -- Uncommon.
				--GameTooltip:AddLine(textColors.green.."GearScore: "..unitScore,.12,1,0);
			--elseif (unitScore >=4300) and (unitScore < 5100) then -- Rare.
				--GameTooltip:AddLine(textColors.blue.."GearScore: "..unitScore,0,.44,.87);
			--elseif (unitScore >= 5100) and (unitScore < 5500) then -- Epic
				--GameTooltip:AddLine(textColors.purple.."GearScore: "..unitScore,.64,.21,.93);
			--elseif (unitScore >= 5500) then -- Legendary
				--GameTooltip:AddLine(textColors.orange.."GearScore: "..unitScore,1,.5,0);
			--end
		--end
		if (ScoreMeSettings["ilvlOn"]) and (unitilvl > 1) then -- Adds ScoreMe & color coding to tooltip.	
			GameTooltip:AddLine(textColors.green.."ScoreMe: "..unitilvl,1,1,1);
		end
		if (boa) and (ScoreMeSettings["boaOn"]) then -- Adds BoA line to tooltip.
			GameTooltip:AddLine(textColors.red.."Player is using BoA gear.",1,1,1);
		end
		if (missing) and (ScoreMeSettings["missingOn"]) then -- Adds missing gear to tooltip.
			GameTooltip:AddLine(textColors.red.."Some items may be missing.",1,1,1);
		end
		if (pvp) and (ScoreMeSettings["pvpOn"]) then -- Adds missing gear to tooltip.
			GameTooltip:AddLine(textColors.red.."Player has PVP gear equipped.",1,1,1);
		end
	--end 
end

-- Setup to pass unit item to ScoreMeItemTip function.
function ScoreMePreSetItem()
	_, iLink = GameTooltip:GetItem(); 
	ScoreMeItemTip(iLink, GameTooltip);
end

-- Setup to pass linked item to ScoreMeItemTip function.
function ScoreMePreRefItem()
	_, iLink = ItemRefTooltip:GetItem(); 
	ScoreMeItemTip(iLink, ItemRefTooltip);
end

-- Setup to pass compared item 1 to ScoreMeItemTip function.
function ScoreMePreCompareItem1() 
	_, iLink = ShoppingTooltip1:GetItem(); 
	ScoreMeItemTip(iLink, ShoppingTooltip1);
end

-- Setup to pass compared item 2 to ScoreMeItemTip function.
function ScoreMePreCompareItem2() 
	_, iLink = ShoppingTooltip2:GetItem(); 
	ScoreMeItemTip(iLink, ShoppingTooltip2); 
end

-- Adds GearScore to the current tooltip.
function ScoreMeItemTip(iLink, tip)
	if (combatStatus) or not (IsEquippableItem(iLink)) then
		return;
	end
	local _, targetClass = UnitClass("player");
	local _, _, _, _, _, _, _, _, itemEquip = GetItemInfo(iLink);
	local tempScore = GetItemScore(iLink);
	if (tempScore > 0) then
		if  (targetClass == "HUNTER") then -- Adjusts GearScore weights for Hunters.
			if (itemEquip == "INVTYPE_RANGEDRIGHT") or (itemEquip == "INVTYPE_RANGED") then
				tempScore = floor(tempScore * 6.321);
			elseif (itemEquip == "INVTYPE_2HWEAPON") or (itemEquip == "INVTYPE_WEAPONMAINHAND") or (itemEquip == "INVTYPE_WEAPONOFFHAND") or (itemEquip == "INVTYPE_WEAPON") or (itemEquip == "INVTYPE_HOLDABLE") then
				tempScore = floor(tempScore * .281);
			end
		end
		if not(ScoreMeSettings["ScoreMeDisabled"]) and (ScoreMeSettings["itemOn"]) then
			--tip:AddLine("GearScore: "..tempScore,1,1,1); -- Delete "--" to activate GearScore Tooltip feature.
		end
	end
end

-------------------------
-- Extra stuff needed. --
-------------------------

-- Event registration.

local frame = CreateFrame("Frame", "ScoreMe", UIParent);
frame:SetScript("OnEvent", ScoreMeEvents);
frame:RegisterEvent("ADDON_LOADED");
frame:RegisterEvent("PLAYER_REGEN_ENABLED");
frame:RegisterEvent("PLAYER_REGEN_DISABLED");

-- Tooltip integration.
GameTooltip:HookScript("OnTooltipSetUnit", ScoreMeSetUnit) -- Player mouse over tooltip.
GameTooltip:HookScript("OnTooltipSetItem", ScoreMePreSetItem) -- Item mouse over tooltip.
ItemRefTooltip:HookScript("OnTooltipSetItem", ScoreMePreRefItem) -- Linked item tooltip.
ShoppingTooltip1:HookScript("OnTooltipSetItem", ScoreMePreCompareItem1) -- Compare item 1 tooltip.
ShoppingTooltip2:HookScript("OnTooltipSetItem", ScoreMePreCompareItem2) -- Compare item 2 tooltip.

-- Command reference.
SlashCmdList["ScoreMeCmdList"] = UserCommands;
SLASH_ScoreMeCmdList1 = "/ScoreMe";
SLASH_ScoreMeCmdList2 = "/sm";
SLASH_ScoreMeCmdList3 = "/scoreme";